unit WebModuleUnit5;

interface

uses System.SysUtils, System.Classes, Web.HTTPApp,
     datamodule, DB,
     VCL.FlexCel.Core, FlexCel.XlsAdapter, FlexCel.Report, FlexCel.Render;

type
  TWebModule1 = class(TWebModule)
    procedure WebModule1DefaultHandlerAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WebModule1DetailHandlerAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
  private
    procedure RunReport(const template: string; const dataset: TDataSet; const date: TDateTime);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  WebModuleClass: TComponentClass = TWebModule1;

implementation

{%CLASSGROUP 'Vcl.Controls.TControl'}

{$R *.dfm}

procedure TWebModule1.WebModule1DefaultHandlerAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
begin
  RunReport('why-master.xls', datamodule1.data, TDateTime(0));
end;

procedure TWebModule1.WebModule1DetailHandlerAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  date: TDateTime;
  day: string;
begin
  day:= Request.QueryFields.Values['day'];
  date := Trunc(TFlxDateTime.FromOADate(StrToInt(day), false));
  DataModule1.detail.Parameters.ParamByName('day').Value := date;

  RunReport('why-detail.xls', datamodule1.detail, date);
end;

procedure TWebModule1.RunReport(const template: string; const dataset: TDataSet; const date: TDateTime);
var
  Report: TFlexCelReport;
  Xls: TExcelFile;
  Ts: TMemoryStream;
  Writer: TStreamWriter;
  Html: TFlexCelHtmlExport;
  HtmlState: TPartialExportState;
begin
  Response.SetCustomHeader('Access-Control-Allow-Origin', '*');
  Response.SetCustomHeader('Access-Control-Allow-Headers', 'X-Custom-Header, Cache-Control');

  Report := TFlexCelReport.Create(true);
  try
    Report.AddTable('data', dataset);
    Report.SetValue('day', date);

    Xls := TXlsFile.Create('..\..\' + template, true);
    try
      Ts := TMemoryStream.Create();
      try
        Writer := TStreamWriter.Create(Ts, TEncoding.UTF8);
        try
          Report.Run(Xls);
          Html := TFlexCelHtmlExport.Create(Xls, true);
          try
            Html.HtmlVersion := THtmlVersion.Html_5;
            Html.EmbedImages := true;
            Html.ImageResolution := 192;
            html.BaseUrl := 'http://tmsexample.com/';
            HtmlState := TPartialExportState.Create(TFlexCelWriter.Null, '');
            try
              Html.PartialExportAdd(HtmlState, 'report.html', '', true);
              //We will encode the css and body in the same Response.
              HtmlState.SaveCss(Writer);
              Writer.Write(#0#0#0#0); //Use 4 character 0 as separator between CSS and Body.
              HtmlState.SaveBody(Writer, 1, '');
            finally
              HtmlState.Free;
            end;
          finally
            Html.Free;
          end;

          Response.ContentStream := Ts;
        finally
          Writer.Free;
        end;
      finally
      //don't free the content stream
       // Ts.Free;
      end;
    finally
      Xls.Free;
    end;
  finally
    Report.Free;
  end;

end;


end.
